//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 1999  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// Typist class - creates the main program and links everything
// together
//
// Example applet tag:
// <APPLET CODE="Typist" WIDTH=100 HEIGHT=50>
// 	<PARAM NAME="kbtype" VALUE="US">
// 	<PARAM NAME="series" VALUE="T,R,Q,V,U,D,M,S,N">
// 	<PARAM NAME="buttonlabel" VALUE="Run Typist">
// </APPLET>
//
// Add a <PARAM NAME="realkbtype" VALUE="xx"> entry to enable keymap
// translations.  Example:
// 	<PARAM NAME="kbtype" VALUE="Dvorak">
//	<PARAM NAME="realkbtype" VALUE="US">
//
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.StringTokenizer;

/**
 * This is the main class for the Typist tutor.
 */
public class Typist extends Applet
		implements ActionListener {

	// keyboard type to use, run button text, and command line
	// flags for the application form of the program
	private String	kbtype		= "US";		// application/applet
	private String	real_kbtype	= "US";		// application/applet
	private String	series		= "T,R,Q,V,U,D,M,S,N";
							// application/applet
	private String	buttonlabel	= "Run Typist";	// applet
	private boolean	kflag		= false;	// application/applet
	private boolean	rflag		= false;	// application/applet
	private boolean	sflag		= false;	// application/applet
	private boolean	mflag		= false;	// application/applet
	private boolean	dflag		= false;	// application
	private boolean	hflag		= false;	// application
	private boolean	vflag		= false;	// application

	// keyboard layout, created from the kbtype passed in
	private KBLayout	kblayout	= null;

	// translation keyboard layout, created from the realkbtype passed in
	private KBLayout	real_kblayout	= null;

	// a GUI that indicate startup progress
	private InitProgressGUI	initprogress	= null;

	// usage message, help, and version for the application
	private final String	TRY_USAGE =
	"Try 'Typist -h' for more information."; 
	private final String	USAGE =
	  "Usage: Typist [ options ]\n\n"
	+ "    -k, -kb KBT       Set the keyboard type.  Valid values are US, UK, Dvorak,\n"
	+ "                      DvorakANSI, Microsoft, and others added locally.\n" 
	+ "                      The default value is '" + kbtype + "'.\n"
	+ "    -r, -realkb KBT   Set the actual keyboard type the system has.  Setting\n"
	+ "                      this value makes the program translate key presses on a\n"
	+ "                      keyboard of this type into the equivalent values on the\n"
	+ "                      type given with the -k or -kb options.  Valid values are\n"
	+ "                      as for -k, -kb.  If this option is not used, the program\n"
	+ "                      will not attempt to translate key presses.\n"
	+ "    -s, -series LIST  Offer these series.  LIST is a comma-separated list of\n"
	+ "                      lesson series IDs, and may include locally added series.\n"
	+ "                      The default value is '" + series + "'.\n"
	+ "    -m, -messages     Print startup messages while initializing.\n"
	+ "    -d, -doc          Print out the Typist User's Guide in HTML.\n"
	+ "    -h, -help         Print this message and exit.\n"
	+ "    -v, -version      Output version information and exit.\n";
	private final String	VERSION =
	  "Typist v3.1c (Java), Copyright (C) 1999  Simon Baldwin\n"
	+ "This program comes with ABSOLUTELY NO WARRANTY; for details\n"
	+ "please see the file 'COPYING' supplied with the source code.\n"
	+ "This is free software, and you are welcome to redistribute it\n"
	+ "under certain conditions; again, see 'COPYING' for details.\n"
	+ "This program is released under the GNU General Public License.\n";


	/*
	 * Parse command line arguments into storage for the application.
	 */
	private void parse_args( String args[] ) {
		for ( int index = 0; index < args.length; index++ ) {
			if ( args[ index ].equals( "-k" )
					|| args[ index ].equals( "-kb" )
					|| args[ index ].equals( "-kbtype" )) {
				kflag = true;
				if ( index < args.length - 1 ) {
					kbtype = args[ ++index ];
					kblayout = (KBLayout)ObjectFactory.create_object(
							"KBLayout"  + kbtype );
					if ( kblayout == null ) {
						System.err.println(
						"Typist: invalid keyboard type " + kbtype );
						System.err.println( TRY_USAGE );
						System.exit( 1 );
					}
				}
				else {
					System.err.println(
					"Typist: option requires an argument -- k" );
					System.err.println( TRY_USAGE );
					System.exit( 1 );
				}
			}
			else if ( args[ index ].equals( "-r" )
					|| args[ index ].equals( "-realkb" )
					|| args[ index ].equals( "-realkbtype" )) {
				rflag = true;
				if ( index < args.length - 1 ) {
					real_kbtype = args[ ++index ];
					real_kblayout = (KBLayout)ObjectFactory.create_object(
							"KBLayout"  + real_kbtype );
					if ( real_kblayout == null ) {
						System.err.println(
						"Typist: invalid keyboard type " + real_kbtype );
						System.err.println( TRY_USAGE );
						System.exit( 1 );
					}
				}
				else {
					System.err.println(
					"Typist: option requires an argument -- k" );
					System.err.println( TRY_USAGE );
					System.exit( 1 );
				}
			}
			else if ( args[ index ].equals( "-s" )
					|| args[ index ].equals( "-series" )) {
				sflag = true;
				if ( index < args.length - 1 ) {
					series = args[ ++index ];
					if ( series.length() == 0 ) {
						System.err.println(
					"Typist: can't work without a series" );
						System.err.println( TRY_USAGE );
						System.exit( 1 );
					}
				}
				else {
					System.err.println(
					"Typist: option requires an argument -- s" );
					System.err.println( TRY_USAGE );
					System.exit( 1 );
				}
			}
			else if ( args[ index ].equals( "-m" )
					|| args[ index ].equals( "-msgs" )
					|| args[ index ].equals( "-messages" ))
				mflag = true;
			else if ( args[ index ].equals( "-d" )
					|| args[ index ].equals( "-doc" )
					|| args[ index ].equals( "-documentation" ))
				dflag = true;
			else if ( args[ index ].equals( "-h" )
					|| args[ index ].equals( "-help" ))
				hflag = true;
			else if ( args[ index ].equals( "-v" )
					|| args[ index ].equals( "-version" ))
				vflag = true;
			else {
				System.err.println( "Typist: illegal option -- "
						+ args[ index ]);
				System.err.println( TRY_USAGE );
				System.exit( 1 );
			}
		}

		// if no kbtype was given then default it
		if ( kblayout == null )
			kblayout = (KBLayout)ObjectFactory.create_object(
					"KBLayout" + kbtype );
	}


	/**
	 * Initialize the applet version of the program.
	 * <p>
	 * This is the main entry point for the Typist applet.
	 */
	public void init() {

		// get the parameters - if no kbtype or invalid, just
		// default to initialized value (US)
		String param = getParameter( "kbtype" );
		if ( param != null ) {
			kflag = true;
			kblayout = (KBLayout)ObjectFactory.create_object(
					"KBLayout" + param );
			if ( kblayout != null )
				kbtype = param;
			else {
				System.out.println(
					"Typist: invalid kbtype parameter" );
				System.out.println(
					"Typist: defaulting to type " + kbtype );
				kblayout = (KBLayout)ObjectFactory.create_object(
						"KBLayout" + kbtype );
			}
		}
		// if no realkbtype or invalid, don't instantiate anything
		param = getParameter( "realkbtype" );
		if ( param != null ) {
			rflag = true;
			real_kblayout = (KBLayout)ObjectFactory.create_object(
					"KBLayout" + param );
			if ( real_kblayout != null )
				real_kbtype = param;
			else {
				System.out.println(
					"Typist: invalid realkbtype parameter" );
				System.out.println(
					"Typist: parameter ignored" );
			}
		}
		param = getParameter( "buttonlabel" );
		if ( param != null )
			buttonlabel = param;
		param = getParameter( "series" );
		if ( param != null && param.length() > 0 ) {
			sflag = true;
			series = param;
		}
		param = getParameter( "messages" );
		if ( param != null )
			mflag = true;

		// if no kbtype was given then default it
		if ( kblayout == null )
			kblayout = (KBLayout)ObjectFactory.create_object(
					"KBLayout" + kbtype );

		// create a get-it-all-going button as the only part
		// of the applet - necessary since the main code makes
		// heavy use of frames and menubars
		Button run_button = new Button( buttonlabel );
		run_button.addActionListener( this );
		setLayout( new FlowLayout( FlowLayout.CENTER, 0, 0 ) );
		add( run_button );
	}


	/*
	 * Convenience function to show some startup status where the
	 * run up to getting the display on the screen is a slow one.
	 */
	private void show_status( boolean inAnApplet, String status ) {

		// expected number of checkpoint calls that we will make
		// to the init progress indicator, based on how many
		// calls we get
		final int INIT_CHECKPOINTS	= 10;

		// if this is the first call then put up a 'starting up' frame,
		// the wait a while to let it render
		if ( initprogress == null ) {
			initprogress = new InitProgressGUI( inAnApplet,
				INIT_CHECKPOINTS,
				"Typist v3.1 is starting, please wait..." );
			initprogress.pack();
			initprogress.setVisible( true );
			try { Thread.sleep( 200 ); }
			catch ( InterruptedException e ) {}
		}

		// checkpoint the progress indicator
		initprogress.checkpoint();
		try { Thread.sleep( 200 ); }
		catch ( InterruptedException e ) {}

		// finally just print out the message to stdout
		if ( mflag )
			if ( inAnApplet )
				showStatus( status );
			else
				System.out.println( status );
	}


	/*
	 * Start up all the frames of the real program - called by either
	 * the action routine of the applet's run button, or by main in
	 * the application.
	 */
	private void start_frames( boolean inAnApplet ) {

		// create the control window, but keep it hidden for
		// the moment
		show_status( inAnApplet, "Creating Control GUI..." );
		ControlGUI control = new ControlGUI( inAnApplet );

		// add lesson series to the control window
		show_status( inAnApplet, "Adding in the lesson series..." );
		StringTokenizer st = new StringTokenizer( series, " \t,:" );
		while ( st.hasMoreTokens() ) {
			String series_id = st.nextToken();
			control.add_series( series_id );
		}

		// create the sub-windows
		show_status( inAnApplet, "Creating Tutorial GUI..." );
		TutorialGUI tutorial = new TutorialGUI( inAnApplet );
		tutorial.pack(); tutorial.setVisible( false );

		show_status( inAnApplet, "Generating keyboard statistics ("
				+ kbtype + ")..." );
		KBStatistics kbstats = new KBStatistics( kblayout );
		show_status( inAnApplet, "Creating Statistics GUI..." );
		StatisticsGUI statistics = new StatisticsGUI( inAnApplet,
				kbstats );
		statistics.pack(); statistics.setVisible( false );

		show_status( inAnApplet, "Creating Keyboard GUI..." );
		KeyboardGUI keyboard = new KeyboardGUI( inAnApplet, kblayout );
		keyboard.pack(); keyboard.setVisible( false );

		show_status( inAnApplet, "Creating Exercise GUI..." );
		ExerciseGUI exercise = new ExerciseGUI( inAnApplet, kbstats );
		exercise.pack(); exercise.setVisible( false );

		show_status( inAnApplet, "Creating Help GUI..." );
		HelpGUI help = new HelpGUI( inAnApplet );
		help.pack(); help.setVisible( false );

		show_status( inAnApplet, "Creating About dialog..." );
		AboutGUI about = new AboutGUI( inAnApplet, control, false );
		about.pack(); about.setVisible( false );

		// register all the sub-windows with the control window
		show_status( inAnApplet, "Linking the GUIs together..." );
		exercise.register_statistics( statistics );
		exercise.register_keyboard( keyboard );
		control.register_about( about );
		control.register_tutorial( tutorial );
		control.register_exercise( exercise );
		control.register_statistics( statistics );
		control.register_keyboard( keyboard );
		control.register_help( help );

		// if we have to do keymap translations, add a translator to
		// the exercise GUI
		if ( real_kblayout != null )
			exercise.register_translator(
				new KBTranslator( real_kblayout, kblayout ));

		// finally reveal the control window
		show_status( inAnApplet, "Typist initialization complete" );
		control.pack(); control.setVisible( true );

		// if in applet mode, clear the status line
		if ( inAnApplet )
			show_status( inAnApplet, "" );

		// dispose of any initialization progress indicator
		if ( initprogress != null ) {
			initprogress.dispose();
			initprogress = null;
		}
	}


	/*
 	 * Handle the GUI actions in the applet.
	 */
	public void actionPerformed( ActionEvent evt ) {
		if ( evt.getSource() instanceof Button ) {

			// must be the run button, so start up the
			// frames and stuff, then sit back and watch
			// the antics
			start_frames( true );
		}
	}


	/**
	 * Parse the command line arguments, create frames, and run
	 * the application.
	 * <p>
	 * This is the main entry point for the Typist application.
	 */
	public static void main( String args[] ) {

		// parse the arguments
		Typist typist = new Typist();
		typist.parse_args( args );

		// do the easy stuff with doc, help and version first
		if ( typist.dflag ) {
			System.out.println( HelpHTML.toHTML( HelpText.data ));
			System.exit( 0 );
		}
		if ( typist.hflag ) {
			System.err.println( typist.USAGE );
			System.exit( 0 );
		}
		if ( typist.vflag ) {
			System.err.println( typist.VERSION );
			System.exit( 0 );
		}

		// create the frames set (no need for the intermediate
		// applet "run" button here)
		typist.start_frames( false );
	}
}
